from localstack.utils.aws import aws_models
fTjMA=super
fTjMc=None
fTjMz=id
class LambdaLayer(aws_models.LambdaFunction):
 def __init__(self,arn):
  fTjMA(LambdaLayer,self).__init__(arn)
  self.cwd=fTjMc
  self.runtime=''
  self.handler=''
  self.envvars={}
  self.versions={}
class BaseComponent(aws_models.Component):
 def name(self):
  return self.fTjMz.split(':')[-1]
class RDSDatabase(BaseComponent):
 def __init__(self,fTjMz,env=fTjMc):
  fTjMA(RDSDatabase,self).__init__(fTjMz,env=env)
class RDSCluster(BaseComponent):
 def __init__(self,fTjMz,env=fTjMc):
  fTjMA(RDSCluster,self).__init__(fTjMz,env=env)
class AppSyncAPI(BaseComponent):
 def __init__(self,fTjMz,env=fTjMc):
  fTjMA(AppSyncAPI,self).__init__(fTjMz,env=env)
class AmplifyApp(BaseComponent):
 def __init__(self,fTjMz,env=fTjMc):
  fTjMA(AmplifyApp,self).__init__(fTjMz,env=env)
class ElastiCacheCluster(BaseComponent):
 def __init__(self,fTjMz,env=fTjMc):
  fTjMA(ElastiCacheCluster,self).__init__(fTjMz,env=env)
class TransferServer(BaseComponent):
 def __init__(self,fTjMz,env=fTjMc):
  fTjMA(TransferServer,self).__init__(fTjMz,env=env)
class CloudFrontDistribution(BaseComponent):
 def __init__(self,fTjMz,env=fTjMc):
  fTjMA(CloudFrontDistribution,self).__init__(fTjMz,env=env)
class CodeCommitRepository(BaseComponent):
 def __init__(self,fTjMz,env=fTjMc):
  fTjMA(CodeCommitRepository,self).__init__(fTjMz,env=env)
# Created by pyminifier (https://github.com/liftoff/pyminifier)



import os
fsbPu=False
fsbPO=True
fsbPh=Exception
import logging
import traceback
import localstack
from localstack.constants import LOCALSTACK_ROOT_FOLDER
from localstack.utils.common import(mkdir,run,new_tmp_file,rm_rf,chmod_r,download,unzip,get_arch,in_docker,is_alpine,now,is_command_available)
from localstack_ext.constants import ARTIFACTS_REPO
LOG=logging.getLogger(__name__)
RULE_ENGINE_INSTALL_URL='https://github.com/whummer/serverless-iot-offline'
H2_DOWNLOAD_URL='http://www.h2database.com/h2-2019-10-14.zip'
REDIS_URL_PATTERN='https://github.com/whummer/miniredis/raw/master/build/miniredis.<arch>.bin'
SSL_CERT_URL='%s/raw/master/local-certs/server.key'%ARTIFACTS_REPO
MINIREDIS_BINARY=os.path.join(LOCALSTACK_ROOT_FOLDER,'localstack','infra','redis','miniredis.<arch>.bin')
INFRA_DIR=os.path.join(os.path.dirname(localstack.__file__),'infra')
LOCALSTACK_DIR=os.path.dirname(localstack.__file__)
def install_libs():
 install_iot_rule_engine()
 install_postgres()
 install_timescaledb()
 install_redis()
 install_mqtt()
def install_iot_rule_engine():
 target_dir=LOCALSTACK_DIR
 main_file=os.path.join(target_dir,'node_modules','serverless-iot-offline','query.js')
 if not os.path.exists(main_file):
  LOG.info('Installing IoT rule engine. This may take a while.')
  run('cd %s; npm install %s'%(target_dir,RULE_ENGINE_INSTALL_URL))
 return main_file
def install_postgres():
 if not in_docker():
  return
 check_or_install('postgres','postgresql','RDS')
 check_or_install('pg_config','postgresql-dev','RDS')
 if not is_alpine():
  return
 plpython_lib='/usr/local/lib/postgresql/plpython3.so'
 if os.path.exists(plpython_lib):
  return
 run('apk add postgresql-plpython3')
 ext_source='/usr/share/postgresql/extension'
 ext_target='/usr/local/share/postgresql/extension'
 links={plpython_lib:'/usr/lib/postgresql/plpython3.so','%s/plpython3u.control'%ext_target:'%s/plpython3u.control'%ext_source,'%s/plpython3u--1.0.sql'%ext_target:'%s/plpython3u--1.0.sql'%ext_source,'%s/plpython3u--unpackaged--1.0.sql'%ext_target:'%s/plpython3u--unpackaged--1.0.sql'%ext_source}
 for target,source in links.items():
  if os.path.exists(source)and not os.path.exists(target):
   mkdir(os.path.dirname(target))
   run('ln -s %s %s'%(source,target))
def install_timescaledb():
 if not in_docker():
  return
 if os.path.exists('/usr/lib/postgresql/timescaledb.so'):
  return
 check_or_install('gcc','cmake gcc git musl-dev libc-dev','Timestream')
 ts_dir='/tmp/timescaledb'
 tag='2.0.0-rc4'
 run('cd /tmp; git clone https://github.com/timescale/timescaledb.git')
 run('cd %s; git checkout %s; ./bootstrap -DREGRESS_CHECKS=OFF; cd build; make; make install'%(ts_dir,tag))
 rm_rf('/tmp/timescaledb')
def install_redis():
 check_or_install('redis-server','redis','ElastiCache')
 return 'redis-server'
def install_mqtt():
 check_or_install('mosquitto','mosquitto','IoT')
 return 'mosquitto'
def install_alpine_package(package,api_name):
 if not is_alpine():
  return
 LOG.info('Downloading dependencies for %s API. This may take a while.'%api_name)
 run('apk add %s'%package)
def check_or_install(command,package,api):
 if not is_command_available(command):
  install_alpine_package(package,api)
def setup_ssl_cert():
 from localstack.services import generic_proxy
 target_file=generic_proxy.get_cert_pem_file_path()
 cache_duration_secs=6*60*60
 if os.path.exists(target_file):
  mod_time=os.path.getmtime(target_file)
  if mod_time>(now()-cache_duration_secs):
   return
 download_github_artifact(SSL_CERT_URL,target_file)
def download_github_artifact(url,target_file):
 def do_download(url,print_error=fsbPu):
  try:
   download(url,target_file)
   return fsbPO
  except fsbPh as e:
   if print_error:
    LOG.info('Unable to download local test SSL certificate from %s to %s: %s %s'%(url,target_file,e,traceback.format_exc()))
 result=do_download(url)
 if not result:
  url=url.replace('https://github.com','https://cdn.jsdelivr.net/gh')
  url=url.replace('/raw/master/','@master/')
  do_download(url,fsbPO)
def install_h2():
 target_dir=os.path.join(INFRA_DIR,'h2')
 if not os.path.exists(target_dir):
  mkdir(target_dir)
  zip_file=new_tmp_file()
  LOG.info('Downloading dependencies for RDS server. This may take a while.')
  download(H2_DOWNLOAD_URL,zip_file)
  unzip(zip_file,target_dir)
  rm_rf(zip_file)
def install_miniredis():
 arch=get_arch()
 bin_path=MINIREDIS_BINARY.replace('<arch>',arch)
 if not os.path.exists(bin_path):
  redis_folder=os.path.dirname(bin_path)
  mkdir(redis_folder)
  url=REDIS_URL_PATTERN.replace('<arch>',arch)
  LOG.debug('Downloading binary from %s'%url)
  download(url,bin_path)
  chmod_r(bin_path,0o755)
 return bin_path
# Created by pyminifier (https://github.com/liftoff/pyminifier)



import os
LBcNm=object
LBcNR=None
LBcNV=Exception
LBcND=set
LBcNp=property
LBcNk=classmethod
LBcNY=True
LBcNQ=getattr
LBcNq=type
LBcNa=isinstance
LBcNn=list
LBcNh=False
import re
import json
import logging
import yaml
import requests
from dulwich import porcelain
from dulwich.repo import Repo
from dulwich.client import get_transport_and_path_from_url
from localstack import config
from localstack.constants import API_ENDPOINT
from localstack.utils.common import(load_file,to_str,to_bytes,mkdir,save_file,cp_r,run,is_command_available,clone,retry,safe_requests,disk_usage)
from localstack_ext.bootstrap.licensing import get_auth_headers
LOG=logging.getLogger(__name__)
PERISTED_FOLDERS=['api_states','dynamodb','kinesis']
class CloudPodManager(LBcNm):
 BACKEND='_none_'
 def __init__(self,pod_name=LBcNR,config=LBcNR):
  self.pod_name=pod_name
  self._pod_config=config
 def push(self):
  raise LBcNV('Not implemented')
 def pull(self):
  raise LBcNV('Not implemented')
 def get_pod_size(self):
  raise LBcNV('Not implemented')
 def restart_container(self):
  LOG.info('Restarting LocalStack instance with updated persistence state - this may take some time ...')
  data={'action':'restart'}
  url='%s/health'%config.get_edge_url()
  try:
   requests.post(url,data=json.dumps(data))
  except requests.exceptions.ConnectionError:
   pass
  def check_status():
   LOG.info('Waiting for LocalStack instance to be fully initialized ...')
   response=requests.get(url)
   content=json.loads(to_str(response.content))
   statuses=[v for k,v in content['services'].items()]
   assert LBcND(statuses)==LBcND(['running'])
  retry(check_status,sleep=3,retries=10)
 @LBcNp
 def pod_config(self):
  return self._pod_config or PodConfigManager.pod_config(self.pod_name)
 @LBcNk
 def get(cls,pod_name):
  pod_config=PodConfigManager.pod_config(pod_name)
  backend=pod_config.get('backend')
  for clazz in cls.__subclasses__():
   if clazz.BACKEND==backend:
    return clazz(pod_name=pod_name,config=pod_config)
  raise LBcNV('Unable to find Cloud Pod manager implementation type "%s"'%backend)
 @LBcNk
 def data_dir(cls):
  if not config.DATA_DIR:
   try:
    details=run('%s inspect %s'%(config.DOCKER_CMD,config.MAIN_CONTAINER_NAME))
    details=json.loads(to_str(details))[0]
    mounts=details.get('Mounts')
    env=details.get('Config',{}).get('Env',[])
    data_dir_env=[e for e in env if e.startswith('DATA_DIR=')][0].partition('=')[2]
    data_dir_host=[m for m in mounts if m['Destination']==data_dir_env][0]['Source']
    data_dir_host=re.sub(r'^(/host_mnt)?',r'',data_dir_host)
    config.DATA_DIR=data_dir_host
   except LBcNV:
    LOG.warning('''Unable to determine DATA_DIR from LocalStack Docker container - please make sure $MAIN_CONTAINER_NAME is configured properly''')
  if not config.DATA_DIR:
   raise LBcNV('Working with local cloud pods requires $DATA_DIR configuration')
  return config.DATA_DIR
class CloudPodManagerGit(CloudPodManager):
 BACKEND='git'
 def push(self):
  repo=self.local_repo()
  client,path=self.client()
  branch=to_bytes(self.pod_config.get('branch'))
  remote_location=self.pod_config.get('url')
  try:
   porcelain.pull(repo,remote_location,refspecs=branch)
  except LBcNV as e:
   LOG.info('Unable to pull repo: %s'%e)
  is_empty_repo=b'HEAD' not in repo or repo.refs.allkeys()==LBcND([b'HEAD'])
  if is_empty_repo:
   LOG.debug('Initializing empty repository %s'%self.clone_dir)
   init_file=os.path.join(self.clone_dir,'.init')
   save_file(init_file,'')
   porcelain.add(repo,init_file)
   porcelain.commit(repo,message='Initial commit')
  if branch not in repo:
   porcelain.branch_create(repo,branch,force=LBcNY)
  self.switch_branch(branch)
  for folder in PERISTED_FOLDERS:
   LOG.info('Copying persistence folder %s to local git repo %s'%(folder,self.clone_dir))
   src_folder=os.path.join(self.data_dir(),folder)
   tgt_folder=os.path.join(self.clone_dir,folder)
   cp_r(src_folder,tgt_folder)
   files=tgt_folder
   if os.path.isdir(files):
    files=[os.path.join(root,f)for root,_,files in os.walk(tgt_folder)for f in files]
   if files:
    porcelain.add(repo,files)
  porcelain.commit(repo,message='Update state')
  porcelain.push(repo,remote_location,branch)
 def pull(self):
  repo=self.local_repo()
  client,path=self.client()
  remote_refs=client.fetch(path,repo)
  branch=self.pod_config.get('branch')
  remote_ref=b'refs/heads/%s'%to_bytes(branch)
  if remote_ref not in remote_refs:
   raise LBcNV('Unable to find branch "%s" in remote git repo'%branch)
  remote_location=self.pod_config.get('url')
  self.switch_branch(branch)
  branch_ref=b'refs/heads/%s'%to_bytes(branch)
  from dulwich.errors import HangupException
  try:
   porcelain.pull(repo,remote_location,branch_ref)
  except HangupException:
   pass
  for folder in PERISTED_FOLDERS:
   src_folder=os.path.join(self.clone_dir,folder)
   tgt_folder=os.path.join(self.data_dir(),folder)
   cp_r(src_folder,tgt_folder,rm_dest_on_conflict=LBcNY)
  self.restart_container()
 def client(self):
  client,path=get_transport_and_path_from_url(self.pod_config.get('url'))
  return client,path
 def local_repo(self):
  self.clone_dir=LBcNQ(self,'clone_dir',LBcNR)
  if not self.clone_dir:
   pod_dir_name=re.sub(r'(\s|/)+','',self.pod_name)
   self.clone_dir=os.path.join(config.TMP_FOLDER,'pods',pod_dir_name,'repo')
   mkdir(self.clone_dir)
   if not os.path.exists(os.path.join(self.clone_dir,'.git')):
    porcelain.clone(self.pod_config.get('url'),self.clone_dir)
  return Repo(self.clone_dir)
 def switch_branch(self,branch):
  repo=self.local_repo()
  if is_command_available('git'):
   return run('cd %s; git checkout %s'%(self.clone_dir,to_str(branch)))
  branch_ref=b'refs/heads/%s'%to_bytes(branch)
  if branch_ref not in repo.refs:
   branch_ref=b'refs/remotes/origin/%s'%to_bytes(branch)
  repo.reset_index(repo[branch_ref].tree)
  repo.refs.set_symbolic_ref(b'HEAD',branch_ref)
 def get_pod_size(self):
  self.local_repo()
  return disk_usage(self.clone_dir)
class PodConfigManagerMeta(LBcNq):
 def __getattr__(cls,attr):
  def _call(*args,**kwargs):
   result=LBcNR
   for manager in cls.CHAIN:
    try:
     tmp=LBcNQ(manager,attr)(*args,**kwargs)
     if tmp:
      if not result:
       result=tmp
      elif LBcNa(tmp,LBcNn)and LBcNa(result,LBcNn):
       result.extend(tmp)
    except LBcNV:
     pass
   if result is not LBcNR:
    return result
   raise LBcNV('Unable to run operation "%s" for local or remote configuration'%attr)
  return _call
class PodConfigManager(LBcNm,metaclass=PodConfigManagerMeta):
 CHAIN=[]
 @LBcNk
 def pod_config(cls,pod_name):
  pods=PodConfigManager.list_pods()
  pod_config=[pod for pod in pods if pod['pod_name']==pod_name]
  if not pod_config:
   raise LBcNV('Unable to find config for pod named "%s"'%pod_name)
  return pod_config[0]
class PodConfigManagerLocal(LBcNm):
 CONFIG_FILE='.localstack.yml'
 def list_pods(self):
  local_pods=self._load_config(safe=LBcNY).get('pods',{})
  local_pods=[{'pod_name':k,'state':'Local Only',**v}for k,v in local_pods.items()]
  existing_names=LBcND([pod['pod_name']for pod in local_pods])
  result=[pod for pod in local_pods if pod['pod_name']not in existing_names]
  return result
 def store_pod_metadata(self,pod_name,metadata):
  pass
 def _load_config(self,safe=LBcNh):
  try:
   return yaml.load(to_str(load_file(self.CONFIG_FILE)))
  except LBcNV:
   if safe:
    return{}
   raise LBcNV('Unable to find and parse config file "%s"'%self.CONFIG_FILE)
class PodConfigManagerRemote(LBcNm):
 def list_pods(self):
  result=[]
  auth_headers=get_auth_headers()
  response=safe_requests.get('%s/cloudpods'%API_ENDPOINT,headers=auth_headers)
  content=response.content
  if response.status_code>=400:
   raise LBcNV('Unable to fetch list of pods from API (code %s): %s'%(response.status_code,content))
  remote_pods=json.loads(to_str(content)).get('cloudpods',[])
  remote_pods=[{'state':'Shared',**pod}for pod in remote_pods]
  result.extend(remote_pods)
  return result
 def store_pod_metadata(self,pod_name,metadata):
  auth_headers=get_auth_headers()
  metadata['pod_name']=pod_name
  response=safe_requests.post('%s/cloudpods'%API_ENDPOINT,json.dumps(metadata),headers=auth_headers)
  content=response.content
  if response.status_code>=400:
   raise LBcNV('Unable to store pod metadata in API (code %s): %s'%(response.status_code,content))
  return json.loads(to_str(content))
PodConfigManager.CHAIN.append(PodConfigManagerLocal())
PodConfigManager.CHAIN.append(PodConfigManagerRemote())
def push_state(pod_name,args):
 backend=CloudPodManager.get(pod_name=pod_name)
 pod_config=clone(backend.pod_config)
 pod_config['size']=backend.get_pod_size()
 PodConfigManager.store_pod_metadata(pod_name,pod_config)
 backend.push()
def pull_state(pod_name,args):
 if not pod_name:
  raise LBcNV('Need to specify a pod name')
 backend=CloudPodManager.get(pod_name=pod_name)
 backend.pull()
def list_pods(args):
 return PodConfigManager.list_pods()
# Created by pyminifier (https://github.com/liftoff/pyminifier)



import re
import os
import six
import subprocess
from localstack import config as localstack_config
from localstack import constants as localstack_constants

FALSE_STRINGS = localstack_constants.FALSE_STRINGS

# api server config
API_PATH = '/v1'
API_PORT_LOCAL = 8183
API_URL = localstack_constants.API_ENDPOINT

# api endpoints
API_PATH_USER = '%s/user' % API_PATH
API_PATH_ORGANIZATIONS = '%s/organizations' % API_PATH
API_PATH_SIGNIN = '%s/signin' % API_PATH_USER
API_PATH_SIGNUP = '%s/signup' % API_PATH_USER
API_PATH_RECOVER = '%s/recover' % API_PATH_USER
API_PATH_RESEND = '%s/resend' % API_PATH_USER
API_PATH_ACTIVATE = '%s/activate' % API_PATH_USER
API_PATH_UNSUBSCRIBE = '%s/unsubscribe' % API_PATH_USER
API_PATH_KEY_ACTIVATE = '%s/activate' % API_PATH
API_PATH_CARDS = '%s/cards' % API_PATH_USER
API_PATH_PLANS = '%s/plans' % API_PATH
API_PATH_FEEDBACKS = '%s/feedbacks' % API_PATH
API_PATH_SUBSCRIPTIONS = '%s/subscriptions' % API_PATH_PLANS
API_PATH_INVOICES = '%s/invoices' % API_PATH_PLANS
API_PATH_CLOUDPODS = '%s/cloudpods' % API_PATH
API_PATH_EVENTS = '%s/events' % API_PATH
API_PATH_STATS = '%s/stats' % API_PATH_EVENTS
API_PATH_GITHUB = '%s/github' % API_PATH
API_PATH_CONFIG = '%s/config' % API_PATH
API_PATH_CI = '%s/ci' % API_PATH
API_PATH_ADMIN = '%s/admin' % API_PATH
API_PATH_STRIPE_WEBHOOK = '%s/webhooks/stripe' % API_PATH

ROOT_FOLDER = os.path.realpath(os.path.join(os.path.dirname(os.path.realpath(__file__)), '..'))

PROTECTED_FOLDERS = ('services', 'utils')

# database connection settings
DB_NAME = 'localstack'
DB_USER = os.environ.get('DB_USER') or 'localstack'
DB_PASS = os.environ.get('DB_PASS')

# localhost IP address and hostname
LOCALHOST_IP = '127.0.0.1'
LOCAL_HOSTNAME = 'localhost.localstack.cloud'

# bind address of local DNS server
DNS_ADDRESS = os.environ.get('DNS_ADDRESS') or '0.0.0.0'

# IP address that AWS endpoints should resolve to in our local DNS server. By default,
# hostnames resolve to 127.0.0.1, which allows to use the LocalStack APIs transparently
# from the host machine. If your code is running in Docker, this should be configured
# to resolve to the Docker bridge network address, e.g., DNS_RESOLVE_IP=172.17.0.1
DNS_RESOLVE_IP = os.environ.get('DNS_RESOLVE_IP') or LOCALHOST_IP

# fallback DNS server to send upstream requests to
DNS_SERVER = os.environ.get('DNS_SERVER')

# SMTP settings (required, e.g., for Cognito)
SMTP_HOST = os.environ.get('SMTP_HOST', '')
SMTP_USER = os.environ.get('SMTP_USER', '')
SMTP_PASS = os.environ.get('SMTP_PASS', '')
SMTP_EMAIL = os.environ.get('SMTP_EMAIL', '')

# whether to automatically start up utility containers (e.g., Spark/Hadoop for EMR, Presto for Athena)
AUTOSTART_UTIL_CONTAINERS = localstack_config.is_env_true('AUTOSTART_UTIL_CONTAINERS')

# optional flags to pass when starting the bigdata Docker container
BIGDATA_DOCKER_FLAGS = os.environ.get('BIGDATA_DOCKER_FLAGS', '')

# Comma-separated list of regex patterns for DNS names to resolve locally.
# Any DNS name not matched against any of the patterns on this whitelist
# will resolve to the real DNS entry, rather than the local one.
DNS_LOCAL_NAME_PATTERNS = os.environ.get('DNS_LOCAL_NAME_PATTERNS', '')

# whether to use static ports and IDs (e.g., cf-<port>) for CloudFormation distributions
CLOUDFRONT_STATIC_PORTS = localstack_config.is_env_true('CLOUDFRONT_STATIC_PORTS')

# whether to enforce IAM policies when processing requests
ENFORCE_IAM = localstack_config.is_env_true('ENFORCE_IAM')

# folder with persistent API backend states
BACKEND_STATES_DIR = os.path.join(localstack_config.DATA_DIR, 'api_states')

# whether to require Pro features and exit with a fault if the API key cannot be activated
REQUIRE_PRO = localstack_config.is_env_true('REQUIRE_PRO')


def use_custom_dns():
    return str(DNS_ADDRESS) not in FALSE_STRINGS


# set USE_SSL to true by default
# TODO fix this!
# localstack_config.USE_SSL = os.environ.get('USE_SSL', '').lower().strip() not in FALSE_STRINGS

# backend service ports
DEFAULT_PORT_LOCAL_DAEMON = 4535
DEFAULT_PORT_LOCAL_DAEMON_ROOT = 4534

# port ranges for service instances (e.g., Postgres DBs, ElastiCache clusters, ...)
SERVICE_INSTANCES_PORTS_START = int(os.environ.get('SERVICE_INSTANCES_PORTS_START') or 4510)
SERVICE_INSTANCES_PORTS_END = int(os.environ.get('SERVICE_INSTANCES_PORTS_END') or
    (SERVICE_INSTANCES_PORTS_START + 30))

# port for Azure APIs
PORT_AZURE = 12121

# add default service ports (TODO needed?)
localstack_constants.DEFAULT_SERVICE_PORTS['azure'] = PORT_AZURE

# Docker host name resolvable from containers
DOCKER_HOST_NAME = 'host.docker.internal'

# Port where Hive/metastore/Spark are available for EMR/Athena
PORT_HIVE_METASTORE = 9083
PORT_HIVE_SERVER = 10000
PORT_SPARK_MASTER = 7077
PORT_SPARK_UI = 4040

if localstack_config.DOCKER_HOST_FROM_CONTAINER == DOCKER_HOST_NAME:
    # special case when we're running tests outside of Docker
    if not localstack_config.in_docker():
        image_name = localstack_constants.DOCKER_IMAGE_NAME
        cmd = "docker run --rm --entrypoint= -it %s bash -c 'ping -c 1 %s'" % (image_name, DOCKER_HOST_NAME)
        try:
            out = subprocess.check_output(cmd, shell=True)
            out = out.decode('utf-8') if isinstance(out, six.binary_type) else out
            ip = re.match(r'PING[^\(]+\(([^\)]+)\).*', out, re.MULTILINE | re.DOTALL)
            ip = ip and ip.group(1)
            if ip:
                localstack_config.DOCKER_HOST_FROM_CONTAINER = ip
        except Exception:
            # Swallow this error - Docker daemon potentially not running?
            pass

# update variable names that need to be passed as arguments to Docker
localstack_config.CONFIG_ENV_VARS += [
    'SMTP_HOST', 'SMTP_USER', 'SMTP_PASS', 'SMTP_EMAIL', 'DNS_SERVER', 'DNS_ADDRESS', 'ENFORCE_IAM',
    'DNS_RESOLVE_IP', 'DNS_LOCAL_NAME_PATTERNS', 'AUTOSTART_UTIL_CONTAINERS', 'CLOUDFRONT_STATIC_PORTS',
    'REQUIRE_PRO', 'AZURE', 'BIGDATA_DOCKER_FLAGS'
]

# re-initialize configs in localstack
localstack_config.populate_configs()



import re
import json
from docopt import docopt
from localstack import config
from localstack.utils.common import truncate, format_bytes
from localstack_ext import constants as ext_constants
from localstack_ext.bootstrap import auth


def cmd_login(argv, args):
    """
Usage:
  localstack login [options]

Options:
  --provider=<>           OAuth provider for login (default: github)
  --username=<>           OAuth username for login
    """
    args.update(docopt(cmd_login.__doc__.strip(), argv=argv))

    try:
        provider = args['--provider'] or 'internal'
        auth.login(provider, args['--username'])
        print('Successfully logged in.')
    except Exception as e:
        raise Exception('Authentication error: %s' % e)


def cmd_logout(argv, args):
    """
Usage:
  localstack logout
    """
    args.update(docopt(cmd_logout.__doc__.strip(), argv=argv))
    auth.logout()
    print('Successfully logged out.')


def cmd_ci(argv, args):
    """
Usage:
  localstack ci <subcommand> [options]

Commands:
  init                  Initialize CI configuration for a new repository
  repos                 List continuous integration repositories

Options:
  --provider=<>         CI provider (default: travis)
  --repo=<>             Repository identifier
  --token=<>            API access token for CI provider
    """
    args.update(docopt(cmd_ci.__doc__.strip(), argv=argv))

    if args['<subcommand>'] == 'repos':
        provider = args['--provider'] or 'travis'
        result = auth.get_ci_repos(provider)
        print(result)
    elif args['<subcommand>'] == 'init':
        provider = opt_params(args, ['--provider']) or 'travis'
        repo, token = mand_params(args, ['--repo', '--token'])
        auth.init_ci_repo(repo, token, provider=provider)
        print(repo, token)


def cmd_config(argv, args):
    """
Usage:
  localstack config [ <key> ]
  localstack config <key> <value>

Arguments:
  <key>             Key of configuration property to read or write
  <value>           Value of configuration property to write
    """
    args.update(docopt(cmd_config.__doc__.strip(), argv=argv))

    if args['<value>']:
        auth.set_user_config(args['<key>'], args['<value>'])
        print('Successfully updated configuration value')
    else:
        result = auth.retrieve_user_config()
        if args['<key>']:
            all_values = result
            result = {}
            for key, value in all_values.items():
                if re.match(args['<key>'], key):
                    result[key] = value
        print(json.dumps(result, indent=4))


def cmd_daemons(argv, args):
    """
Usage:
  localstack daemons [ <subcommand> ]

Commands:
  start        Start local daemon processes
  stop         Stop local daemon processes
    """
    args.update(docopt(cmd_daemons.__doc__.strip(), argv=argv))

    from localstack_ext.bootstrap import local_daemon

    if args['<subcommand>'] in [None, 'start']:
        print('Starting local daemons processes ...')
        local_daemon.start_in_background()
    elif args['<subcommand>'] in ['stop']:
        print('Stopping local daemons processes ...')
        local_daemon.kill_servers()


def cmd_pod(argv, args):
    """
Usage:
  localstack pod <subcommand> [ <name> ]

Commands:
  push <name>      Push the state of a local cloud pod
  pull <name>      Pull the state to launch a cloud pod locally
    """
    args.update(docopt(cmd_pod.__doc__.strip(), argv=argv))
    from localstack_ext.bootstrap import pods_client  # leave import here to avoid dependency conflict

    if args['<subcommand>'] == 'push':
        print('Pushing local persistence state to server ...')
        pods_client.push_state(args['<name>'], args)
    elif args['<subcommand>'] == 'pull':
        print('Pulling persistence state from server ...')
        pods_client.pull_state(args['<name>'], args)
    elif args['<subcommand>'] == 'list':
        print('Fetching list of pods from server ...')
        result = pods_client.list_pods(args)
        # print('result', result)
        headers = {'pod_name': 'Name', 'backend': 'Backend', 'url': 'URL', 'size': 'Size', 'state': 'State'}
        print_table(result, headers=headers, formats={'size': format_bytes})


def cmd_status(argv, args):
    from localstack.utils.cli import print_status
    print_status()
    print('\nExtended Details:')
    print('------------------')
    print('Pro Version:\t\t%s' % ext_constants.VERSION)
    # at this point we can assume that the API key has been validated...
    print('API Key:\t\tvalidated')


def register_commands():
    config.CLI_COMMANDS['daemons'] = {
        'description': 'Manage local daemon processes',
        'parameters': [],
        'function': cmd_daemons
    }
    config.CLI_COMMANDS['pod'] = {
        'description': 'Manage state of local cloud pods',
        'parameters': [],
        'function': cmd_pod
    }
    if 'status' in config.CLI_COMMANDS:
        config.CLI_COMMANDS['status']['function'] = cmd_status

    config.CLI_COMMANDS['login'] = {
        'description': 'Log in using an external OAuth provider (e.g., Github)',
        'function': cmd_login
    }
    config.CLI_COMMANDS['logout'] = {
        'description': 'Log out and delete any session tokens',
        'function': cmd_logout
    }

    # TODO enable commands below?
    return

    config.CLI_COMMANDS['config'] = {
        'command': '  localstack config',
        'description': 'Manage configuration values',
        'parameters': [],
        'function': cmd_config
    }
    config.CLI_COMMANDS['ci'] = {
        'description': 'Manage continuous integration repositories and settings',
        'function': cmd_ci
    }


# -----------------
# HELPER FUNCTIONS
# -----------------

def mand_params(args, names=[]):
    return get_params(args, names, mandatory=True)


def opt_params(args, names=[]):
    return get_params(args, names, mandatory=False)


def get_params(args, names=[], mandatory=True):
    result = ()
    for name in names:
        value = None
        if name in args:
            value = args[name]
        if not value and mandatory:
            raise Exception('Please provide %s=... parameter' % name)
        result += (value,)
    if len(result) == 1:
        return result[0]
    return result


def print_table(data_list, headers=[], formats={}):
    from tabulate import tabulate
    if not isinstance(data_list, list):
        raise Exception('Non-list input cannot be rendered as table: %s' % data_list)
    formats = formats or {}
    data_list = [i if isinstance(i, dict) else {'value': i} for i in data_list]
    # prepare headers
    if isinstance(headers, dict):
        headers = [{'key': k, 'name': v} for k, v in headers.items()]
    # build column metadata
    columns = headers
    if not columns:
        columns = []
        for row in data_list:
            for key in row.keys():
                if key not in columns:
                    columns.append(key)
    for idx in range(len(columns)):
        column = columns[idx]
        columns[idx] = column if isinstance(column, dict) else {'key': column, 'name': column}

    # build rows
    def _format(row, key):
        value = row.get(key)
        formatter = formats.get(key)
        if formatter:
            return formatter(value)
        if not value:
            return value
        return truncate(str(value), max_length=30)
    rows = [[_format(row, h['key']) for h in columns] for row in data_list]
    headers = [col['name'] for col in columns]
    result = tabulate(rows, headers=headers)
    print(result)



import os
tuDNb=Exception
tuDNk=any
tuDNs=str
tuDNL=None
tuDNU=False
tuDNP=bool
tuDNw=True
import logging
from localstack import config as localstack_config
from localstack.utils import common
from localstack.constants import(LOCALSTACK_WEB_PROCESS,LOCALSTACK_INFRA_PROCESS,TRUE_STRINGS)
from localstack.utils.bootstrap import is_api_enabled,API_DEPENDENCIES
from localstack_ext import config as config_ext
from localstack_ext.bootstrap import licensing,cli,install,local_daemon
LOG=logging.getLogger(__name__)
EXTERNAL_PORT_APIS=('apigateway','apigatewayv2','athena','cloudfront','codecommit','ecs','ecr','elasticache','mediastore','rds','transfer','kafka','neptune','azure')
def register_localstack_plugins():
 _setup_logging()
 is_infra_process=os.environ.get(LOCALSTACK_INFRA_PROCESS)in TRUE_STRINGS
 is_api_key_configured=api_key_configured()
 if is_infra_process:
  install.install_libs()
  if is_api_key_configured:
   install.setup_ssl_cert()
 if os.environ.get(LOCALSTACK_WEB_PROCESS):
  return{}
 with licensing.prepare_environment():
  try:
   from localstack_ext.services import dns_server
   dns_server.setup_network_configuration()
  except tuDNb:
   return
  if is_infra_process:
   load_plugin_files()
  try:
   if is_api_key_configured and not is_infra_process and is_api_enabled('ec2'):
    local_daemon.start_in_background()
  except tuDNb as e:
   LOG.warning('Unable to start local daemon process: %s'%e)
  if is_api_key_configured:
   if os.environ.get('EDGE_PORT')and not localstack_config.EDGE_PORT_HTTP:
    LOG.warning(('!! Configuring EDGE_PORT={p} without setting EDGE_PORT_HTTP may lead '+'to issues; better leave the defaults, or set EDGE_PORT=443 and EDGE_PORT_HTTP={p}').format(p=localstack_config.EDGE_PORT))
   else:
    port=localstack_config.EDGE_PORT
    localstack_config.EDGE_PORT=443
    localstack_config.EDGE_PORT_HTTP=port
 API_DEPENDENCIES['apigateway']=['apigatewayv2']
 API_DEPENDENCIES['athena']=['emr']
 API_DEPENDENCIES['docdb']=['rds']
 API_DEPENDENCIES['ecs']=['ecr']
 API_DEPENDENCIES['elasticache']=['ec2']
 API_DEPENDENCIES['elb']=['elbv2']
 API_DEPENDENCIES['emr']=['athena','s3']
 API_DEPENDENCIES['glacier']=['s3']
 API_DEPENDENCIES['glue']=['rds']
 API_DEPENDENCIES['iot']=['iotanalytics','iot-data','iotwireless']
 API_DEPENDENCIES['kinesisanalytics']=['kinesis','dynamodb']
 API_DEPENDENCIES['neptune']=['rds']
 API_DEPENDENCIES['rds']=['rds-data']
 API_DEPENDENCIES['redshift']=['redshift-data']
 API_DEPENDENCIES['timestream']=['timestream-write','timestream-query']
 API_DEPENDENCIES['transfer']=['s3']
 docker_flags=[]
 if config_ext.use_custom_dns():
  if not common.is_port_open(dns_server.DNS_PORT,protocols='tcp'):
   docker_flags+=['-p {a}:{p}:{p}'.format(a=config_ext.DNS_ADDRESS,p=dns_server.DNS_PORT)]
  if not common.is_port_open(dns_server.DNS_PORT,protocols='udp'):
   docker_flags+=['-p {a}:{p}:{p}/udp'.format(a=config_ext.DNS_ADDRESS,p=dns_server.DNS_PORT)]
 if tuDNk([is_api_enabled(api)for api in EXTERNAL_PORT_APIS]):
  docker_flags+=['-p {start}-{end}:{start}-{end}'.format(start=config_ext.SERVICE_INSTANCES_PORTS_START,end=config_ext.SERVICE_INSTANCES_PORTS_END)]
 if is_api_enabled('eks'):
  kube_config=os.path.expanduser('~/.kube/config')
  if os.path.exists(kube_config):
   docker_flags+=['-v %s:/root/.kube/config'%kube_config]
 if is_api_enabled('azure'):
  docker_flags+=['-p {port}:{port}'.format(port=5671)]
 if os.environ.get('AZURE'):
  docker_flags+=['-p {p}:{p}'.format(p=config_ext.PORT_AZURE)]
 result={'docker':{'run_flags':' '.join(docker_flags)}}
 return result
def load_plugin_files():
 try:
  from localstack.services.plugins import register_plugin,Plugin
  from localstack_ext.services import edge
  from localstack_ext.services.amplify import amplify_starter
  from localstack_ext.services.applicationautoscaling import(applicationautoscaling_starter,applicationautoscaling_listener)
  from localstack_ext.services.appsync import appsync_starter
  from localstack_ext.services.azure import azure_starter
  from localstack_ext.services.apigateway import apigateway_extended
  from localstack_ext.services.athena import athena_starter
  from localstack_ext.services.awslambda import lambda_extended
  from localstack_ext.services.batch import batch_starter,batch_listener
  from localstack_ext.services.cloudformation import cloudformation_extended
  from localstack_ext.services.cloudfront import cloudfront_starter
  from localstack_ext.services.cloudtrail import cloudtrail_starter
  from localstack_ext.services.codecommit import codecommit_starter,codecommit_listener
  from localstack_ext.services.cognito import cognito_starter,cognito_listener
  from localstack_ext.services.docdb import docdb_api
  from localstack_ext.services.dynamodb import dynamodb_extended
  from localstack_ext.services.ecr import ecr_starter,ecr_listener
  from localstack_ext.services.ecs import ecs_starter,ecs_listener
  from localstack_ext.services.eks import eks_starter
  from localstack_ext.services.elasticache import elasticache_starter
  from localstack_ext.services.elb import elb_starter,elb_listener
  from localstack_ext.services.emr import emr_starter,emr_listener
  from localstack_ext.services.events import events_extended
  from localstack_ext.services.glacier import glacier_starter,glacier_listener
  from localstack_ext.services.glue import glue_starter,glue_listener
  from localstack_ext.services.iot import iot_starter,iot_listener
  from localstack_ext.services.iam import iam_extended
  from localstack_ext.services.kafka import kafka_starter
  from localstack_ext.services.kinesisanalytics import kinesis_analytics_api
  from localstack_ext.services.kms import kms_starter,kms_listener
  from localstack_ext.services.mediastore import mediastore_starter
  from localstack_ext.services.neptune import neptune_api
  from localstack_ext.services.organizations import organizations_starter
  from localstack_ext.services.qldb import qldb_starter
  from localstack_ext.services.rds import rds_starter,rds_listener
  from localstack_ext.services.redshift import redshift_starter,redshift_listener
  from localstack_ext.services.route53 import route53_extended
  from localstack_ext.services.s3 import s3_extended
  from localstack_ext.services.sagemaker import sagemaker_starter
  from localstack_ext.services.secretsmanager import secretsmanager_extended
  from localstack_ext.services.serverlessrepo import serverlessrepo_starter
  from localstack_ext.services.servicediscovery import servicediscovery_starter
  from localstack_ext.services.ses import ses_extended
  from localstack_ext.services.sns import sns_extended
  from localstack_ext.services.sqs import sqs_extended
  from localstack_ext.services.stepfunctions import stepfunctions_extended
  from localstack_ext.services.sts import sts_extended
  from localstack_ext.services.timestream import timestream_starter
  from localstack_ext.services.transfer import transfer_starter
  from localstack_ext.services.xray import xray_starter,xray_listener
  from localstack_ext.utils import persistence as persistence_ext
  from localstack_ext.utils.aws import aws_utils
  from localstack_ext.bootstrap.dashboard import dashboard_extended
  register_plugin(Plugin('amplify',start=amplify_starter.start_amplify))
  register_plugin(Plugin('application-autoscaling',start=applicationautoscaling_starter.start_applicationautoscaling,listener=applicationautoscaling_listener.UPDATE_APPLICATION_AUTOSCALING))
  register_plugin(Plugin('appsync',start=appsync_starter.start_appsync))
  register_plugin(Plugin('athena',start=athena_starter.start_athena))
  register_plugin(Plugin('azure',start=azure_starter.start_azure))
  register_plugin(Plugin('batch',start=batch_starter.start_batch,listener=batch_listener.UPDATE_BATCH))
  register_plugin(Plugin('cloudfront',start=cloudfront_starter.start_cloudfront))
  register_plugin(Plugin('cloudtrail',start=cloudtrail_starter.start_cloudtrail))
  register_plugin(Plugin('codecommit',start=codecommit_starter.start_codecommit,listener=codecommit_listener.UPDATE_CODECOMMIT))
  register_plugin(Plugin('cognito-identity',start=cognito_starter.start_cognito_identity,listener=cognito_listener.UPDATE_COGNITO_IDENTITY))
  register_plugin(Plugin('cognito-idp',start=cognito_starter.start_cognito_idp,listener=cognito_listener.UPDATE_COGNITO))
  register_plugin(Plugin('docdb',start=docdb_api.start_docdb))
  register_plugin(Plugin('elasticache',start=elasticache_starter.start_elasticache))
  register_plugin(Plugin('elb',start=elb_starter.start_elb,listener=elb_listener.UPDATE_ELB))
  register_plugin(Plugin('elbv2',start=elb_starter.start_elbv2,listener=elb_listener.UPDATE_ELB))
  register_plugin(Plugin('ecr',start=ecr_starter.start_ecr,listener=ecr_listener.UPDATE_ECR))
  register_plugin(Plugin('ecs',start=ecs_starter.start_ecs,listener=ecs_listener.UPDATE_ECS))
  register_plugin(Plugin('eks',start=eks_starter.start_eks))
  register_plugin(Plugin('emr',start=emr_starter.start_emr,listener=emr_listener.UPDATE_EMR))
  register_plugin(Plugin('glacier',start=glacier_starter.start_glacier,listener=glacier_listener.UPDATE_GLACIER))
  register_plugin(Plugin('glue',start=glue_starter.start_glue,listener=glue_listener.UPDATE_GLUE))
  register_plugin(Plugin('iot',start=iot_starter.start_iot,listener=iot_listener.UPDATE_IOT))
  register_plugin(Plugin('kafka',start=kafka_starter.start_kafka))
  register_plugin(Plugin('kinesisanalytics',start=kinesis_analytics_api.start_kinesis_analytics))
  register_plugin(Plugin('kms',start=kms_starter.start_kms,listener=kms_listener.UPDATE_KMS))
  register_plugin(Plugin('mediastore',start=mediastore_starter.start_mediastore))
  register_plugin(Plugin('neptune',start=neptune_api.start_neptune))
  register_plugin(Plugin('organizations',start=organizations_starter.start_organizations))
  register_plugin(Plugin('qldb',start=qldb_starter.start_qldb))
  register_plugin(Plugin('rds',start=rds_starter.start_rds,listener=rds_listener.UPDATE_RDS))
  register_plugin(Plugin('redshift',start=redshift_starter.start_redshift,listener=redshift_listener.UPDATE_REDSHIFT,priority=10))
  register_plugin(Plugin('sagemaker',start=sagemaker_starter.start_sagemaker))
  register_plugin(Plugin('serverlessrepo',start=serverlessrepo_starter.start_serverlessrepo))
  register_plugin(Plugin('servicediscovery',start=servicediscovery_starter.start_servicediscovery))
  register_plugin(Plugin('timestream',start=timestream_starter.start_timestream))
  register_plugin(Plugin('transfer',start=transfer_starter.start_transfer))
  register_plugin(Plugin('xray',start=xray_starter.start_xray,listener=xray_listener.UPDATE_XRAY))
  persistence_ext.enable_extended_persistence()
  lambda_extended.patch_lambda()
  sns_extended.patch_sns()
  sqs_extended.patch_sqs()
  apigateway_extended.patch_apigateway()
  cloudformation_extended.patch_cloudformation()
  stepfunctions_extended.patch_stepfunctions()
  s3_extended.patch_s3()
  iam_extended.patch_iam()
  dynamodb_extended.patch_dynamodb()
  events_extended.patch_events()
  secretsmanager_extended.patch_secretsmanager()
  sts_extended.patch_sts()
  ses_extended.patch_ses()
  route53_extended.patch_route53()
  dashboard_extended.patch_dashboard()
  aws_utils.patch_aws_stack()
  edge.patch_start_edge()
  patch_start_infra()
 except tuDNb as e:
  if 'No module named' not in tuDNs(e):
   print('ERROR: %s'%e)
def patch_start_infra():
 from localstack.services import infra
 try:
  from localstack_ext.utils.aws.metadata_service import start_metadata_service
 except tuDNb:
  start_metadata_service=tuDNL
 def do_start_infra(asynchronous,apis,is_in_docker,*args,**kwargs):
  if common.in_docker():
   try:
    start_metadata_service and start_metadata_service()
   except tuDNb:
    pass
  enforce_before=config_ext.ENFORCE_IAM
  try:
   config_ext.ENFORCE_IAM=tuDNU
   return do_start_infra_orig(asynchronous,apis,is_in_docker,*args,**kwargs)
  finally:
   config_ext.ENFORCE_IAM=enforce_before
 do_start_infra_orig=infra.do_start_infra
 infra.do_start_infra=do_start_infra
def _setup_logging():
 log_level=logging.DEBUG if localstack_config.DEBUG else logging.INFO
 logging.getLogger('localstack_ext').setLevel(log_level)
 logging.getLogger('botocore').setLevel(logging.INFO)
 logging.getLogger('kubernetes').setLevel(logging.INFO)
 logging.getLogger('pyftpdlib').setLevel(logging.INFO)
 logging.getLogger('pyhive').setLevel(logging.INFO)
 logging.getLogger('websockets').setLevel(logging.INFO)
 logging.getLogger('asyncio').setLevel(logging.INFO)
 logging.getLogger('hpack').setLevel(logging.INFO)
 logging.getLogger('jnius.reflect').setLevel(logging.INFO)
 logging.getLogger('dulwich').setLevel(logging.ERROR)
 logging.getLogger('postgresql_proxy').setLevel(logging.INFO)
 logging.getLogger('intercept').setLevel(logging.WARNING)
 logging.getLogger('root').setLevel(logging.WARNING)
 logging.getLogger('').setLevel(logging.WARNING)
def api_key_configured():
 return tuDNP(os.environ.get('LOCALSTACK_API_KEY'))
def register_localstack_commands():
 if api_key_configured():
  cli.register_commands()
 return tuDNw
# Created by pyminifier (https://github.com/liftoff/pyminifier)



#!/usr/bin/env python
GFAzr=dict
GFAzI=super
GFAzg=hasattr
GFAze=Exception
GFAzi=object
GFAzh=dir
GFAzf=None
GFAzV=False
GFAzl=True
GFAzw=len
GFAzb=set
import errno
import socket
from select import select
from socket import(AF_INET,AF_INET6,SOCK_STREAM,IPV6_V6ONLY,SO_REUSEADDR,IPPROTO_TCP,IPPROTO_IPV6,TCP_NODELAY,SOL_SOCKET)
class AttrDict(GFAzr):
 def __init__(self,*args,**kwargs):
  GFAzI(AttrDict,self).__init__(*args,**kwargs)
  self.__dict__=self
def tcp_listen(six,addr,port,blk):
 s=socket.socket(AF_INET6 if six else AF_INET,SOCK_STREAM)
 if six and GFAzg(socket,'IPV6_V6ONLY'):
  s.setsockopt(IPPROTO_IPV6,IPV6_V6ONLY,0)
 s.setsockopt(SOL_SOCKET,SO_REUSEADDR,1)
 s.setsockopt(IPPROTO_TCP,TCP_NODELAY,1)
 s.bind((addr,port))
 s.listen(5)
 s.setblocking(blk)
 return s
def tcp_connect(six,addr,port,blk):
 s=socket.socket(AF_INET6 if six else AF_INET,SOCK_STREAM)
 s.setsockopt(IPPROTO_TCP,TCP_NODELAY,1)
 s.connect((addr,port))
 s.setblocking(blk)
 return s
def safe_close(x):
 try:
  x.close()
 except GFAze:
  pass
class Server(GFAzi):
 def __init__(self,opt,q):
  self.opt=opt
  self.sock=tcp_listen(opt.ip6,opt.bind_addr,opt.bind_port,0)
  self.q=q
 def pre_wait(self,rr,r,w,e):
  r.append(self.sock)
 def post_wait(self,r,w,e):
  if self.sock in r:
   cl,addr=self.sock.accept()
   cl.setblocking(0)
   self.q.append(Proxy(self.opt,cl,addr))
class Half(GFAzi):
 def __init__(self,opt,sock,addr,GFAzh):
  self.opt=opt
  self.sock=sock
  self.addr=addr
  self.GFAzh=GFAzh
  self.name='peer' if self.GFAzh else 'client'
  self.queue=[]
  self.dest=GFAzf
  self.err=GFAzf
  self.ready=GFAzV
 def pre_wait(self,rr,r,w,e):
  if self.ready:
   rr.append(self.sock)
  r.append(self.sock)
  if self.queue:
   w.append(self.sock)
 def post_wait(self,r,w,e):
  if not self.err and self.sock in w and self.queue:
   self.write_some()
  if not self.err and self.sock in r:
   self.ready=GFAzl
   self.copy()
  return self.err
 def error(self,msg,e):
  self.err='Error on %s: %s: %s'%(self.name,msg,e)
  return self.err
 def write_some(self):
  try:
   n=self.sock.send(self.queue[0])
  except GFAze as e:
   return self.error('send error',e)
  if n!=GFAzw(self.queue[0]):
   self.queue[0]=self.queue[0][n:]
  else:
   del self.queue[0]
 def copy(self):
  try:
   buf=self.sock.recv(4096)
  except socket.error as e:
   if e.errno==errno.EWOULDBLOCK:
    self.ready=GFAzV
    return
   return self.error('recv socket error',e)
  except GFAze as e:
   return self.error('recv error',e)
  if GFAzw(buf)==0:
   return self.error('eof',0)
  self.dest.queue.append(buf)
 def close(self):
  safe_close(self.sock)
class Proxy(GFAzi):
 def __init__(self,opt,sock,addr):
  self.opt=opt
  self.cl=Half(opt,sock,addr,'i')
  peer=tcp_connect(opt.ip6,opt.addr,opt.port,0)
  self.peer=Half(opt,peer,addr,'o')
  self.cl.dest=self.peer
  self.peer.dest=self.cl
  self.err=GFAzf
 def pre_wait(self,rr,r,w,e):
  self.cl.pre_wait(rr,r,w,e)
  self.peer.pre_wait(rr,r,w,e)
 def post_wait(self,r,w,e):
  if not self.err:
   self.err=self.cl.post_wait(r,w,e)
  if not self.err:
   self.err=self.peer.post_wait(r,w,e)
  if self.err:
   self.cl.close()
   self.peer.close()
  return self.err
def server_loop(opt):
 qs=[]
 opt['ip6']=opt.get('ip6',GFAzV)
 opt['bind_addr']=opt.get('bind_addr','0.0.0.0')
 opt['addr']=opt.get('addr','127.0.0.1')
 opt['bind_port']=opt.get('bind_port',0)
 opt=AttrDict(opt)
 qs.append(Server(opt,qs))
 while qs:
  rr,r,w,e=[],[],[],[]
  for q in qs:
   q.pre_wait(rr,r,w,e)
  timeo=10.0 if not rr else 0.0
  r,w,e=select(r,w,e,timeo)
  r=GFAzb(r).union(rr)
  for q in qs:
   if q.post_wait(r,w,e):
    qs.remove(q)
def main():
 opt={'ip6':GFAzV,'bind_port':2223,'addr':'192.168.43.191','port':22}
 opt={'ip6':GFAzV,'bind_port':48429,'addr':'128.0.100.3','port':22}
 opt={'ip6':GFAzV,'port':54237,'bind_addr':'128.0.100.2','bind_port':22}
 opt={'ip6':GFAzV,'port':54237,'bind_addr':'192.168.100.3','bind_port':22}
 server_loop(opt)
if __name__=='__main__':
 main()
# Created by pyminifier (https://github.com/liftoff/pyminifier)



import os
bPVNm=Exception
bPVNX=hasattr
from localstack import config
from localstack.utils.aws import aws_stack
from localstack.dashboard import infra as dashboard_infra
from localstack.utils.common import short_uid,run_safe
from localstack.utils.bootstrap import is_api_enabled
from localstack_ext.utils.aws.aws_utils import MARKER_APIGW_REQUEST_REGION,THREAD_LOCAL
from localstack_ext.bootstrap.aws_models import(RDSDatabase,RDSCluster,AppSyncAPI,AmplifyApp,ElastiCacheCluster,TransferServer,CloudFrontDistribution,CodeCommitRepository)
get_graph_orig=dashboard_infra.get_graph
def get_resources(fetch_func):
 try:
  result=[]
  fetch_func(result)
  return result
 except bPVNm:
  pass
 return[]
def get_rds_databases(name_filter,pool,env):
 if not is_api_enabled('rds'):
  return[]
 def fetch_func(result):
  client=aws_stack.connect_to_service('rds')
  dbs=client.describe_db_instances()
  for inst in dbs['DBInstances']:
   obj=RDSDatabase(id=inst['DBInstanceArn'])
   result.append(obj)
 return get_resources(fetch_func)
def get_rds_clusters(name_filter,pool,env):
 if not is_api_enabled('rds'):
  return[]
 def fetch_func(result):
  client=aws_stack.connect_to_service('rds')
  clusters=client.describe_db_clusters()
  for cluster in clusters['DBClusters']:
   obj=RDSCluster(id=cluster['DBClusterArn'])
   result.append(obj)
 return get_resources(fetch_func)
def get_appsync_apis(name_filter,pool,env):
 if not is_api_enabled('appsync'):
  return[]
 def fetch_func(result):
  client=aws_stack.connect_to_service('appsync')
  apis=client.list_graphql_apis()
  for api in apis['graphqlApis']:
   obj=AppSyncAPI(id=api['apiId'])
   result.append(obj)
 return get_resources(fetch_func)
def get_amplify_apps(name_filter,pool,env):
 if not is_api_enabled('amplify'):
  return[]
 def fetch_func(result):
  client=aws_stack.connect_to_service('amplify')
  apps=client.list_apps()
  for app in apps['apps']:
   obj=AmplifyApp(id=app['appId'])
   result.append(obj)
 return get_resources(fetch_func)
def get_elasticache_clusters(name_filter,pool,env):
 if not is_api_enabled('elasticache'):
  return[]
 def fetch_func(result):
  client=aws_stack.connect_to_service('elasticache')
  clusters=client.describe_cache_clusters()
  for cluster in clusters['CacheClusters']:
   obj=ElastiCacheCluster(id=cluster['CacheClusterId'])
   result.append(obj)
 return get_resources(fetch_func)
def get_transfer_servers(name_filter,pool,env):
 if not is_api_enabled('transfer'):
  return[]
 def fetch_func(result):
  client=aws_stack.connect_to_service('transfer')
  servers=client.list_servers()
  for server in servers['Servers']:
   obj=TransferServer(id=server['ServerId'])
   result.append(obj)
 return get_resources(fetch_func)
def get_cloudfront_distributions(name_filter,pool,env):
 if not is_api_enabled('cloudfront'):
  return[]
 def fetch_func(result):
  client=aws_stack.connect_to_service('cloudfront')
  distros=client.list_distributions()
  for distro in distros['DistributionList']['Items']:
   obj=CloudFrontDistribution(id=distro['ARN'])
   result.append(obj)
 return get_resources(fetch_func)
def get_codecommit_repos(name_filter,pool,env):
 if not is_api_enabled('codecommit'):
  return[]
 def fetch_func(result):
  client=aws_stack.connect_to_service('codecommit')
  repos=client.list_repositories()
  for repo in repos['repositories']:
   obj=CodeCommitRepository(id=repo['repositoryId'])
   result.append(obj)
 return get_resources(fetch_func)
def get_graph(*args,**kwargs):
 os.environ['AWS_ACCESS_KEY_ID']=os.environ.get('AWS_ACCESS_KEY_ID')or 'foobar'
 os.environ['AWS_SECRET_ACCESS_KEY']=os.environ.get('AWS_SECRET_ACCESS_KEY')or 'foobar'
 if bPVNX(THREAD_LOCAL,'request_context'):
  region=kwargs.get('region')or config.DEFAULT_REGION
  THREAD_LOCAL.request_context.headers[MARKER_APIGW_REQUEST_REGION]=region
 result=run_safe(get_graph_orig,*args,**kwargs)or{'nodes':[],'edges':[]}
 env=kwargs.get('env')
 name_filter=kwargs.get('name_filter')
 pool={}
 node_ids={}
 databases=get_rds_databases(name_filter,pool=pool,env=env)
 rds_clusters=get_rds_clusters(name_filter,pool=pool,env=env)
 appsync_apis=get_appsync_apis(name_filter,pool=pool,env=env)
 amplify_apps=get_amplify_apps(name_filter,pool=pool,env=env)
 elasticache_clusters=get_elasticache_clusters(name_filter,pool=pool,env=env)
 transfer_servers=get_transfer_servers(name_filter,pool=pool,env=env)
 cloudfront_distributions=get_cloudfront_distributions(name_filter,pool=pool,env=env)
 codecommit_repos=get_codecommit_repos(name_filter,pool=pool,env=env)
 resources={'rds_db':databases,'rds_cluster':rds_clusters,'appsync_api':appsync_apis,'amplify_app':amplify_apps,'elasticache_cluster':elasticache_clusters,'transfer_server':transfer_servers,'cloudfront_distr':cloudfront_distributions,'codecommit_repo':codecommit_repos}
 for res_type,res_list in resources.items():
  for res in res_list:
   uid=short_uid()
   node_ids[res.id]=uid
   result['nodes'].append({'id':uid,'arn':res.id,'name':res.name(),'type':res_type})
 return result
def patch_dashboard():
 dashboard_infra.get_graph=get_graph
# Created by pyminifier (https://github.com/liftoff/pyminifier)



import os
ogjVG=Exception
ogjVA=False
ogjVs=str
ogjVC=int
ogjVn=range
ogjVM=len
ogjVf=object
ogjVQ=True
import re
import sys
import glob
import json
import base64
import logging
import pyaes
from localstack import config as localstack_config
from localstack.utils.common import(safe_requests as requests,load_file,save_file,to_str,to_bytes,md5,parallelize,now_utc,str_insert,str_remove)
from localstack_ext import config
from localstack_ext.config import PROTECTED_FOLDERS,ROOT_FOLDER
from localstack_ext.constants import VERSION
ENV_PREPARED={}
MAX_KEY_CACHE_DURATION_SECS=60*60*24
LOG=logging.getLogger(__name__)
def read_api_key():
 key=os.environ.get('LOCALSTACK_API_KEY')
 if key:
  return key
 raise ogjVG('Unable to retrieve API key. Please configure $LOCALSTACK_API_KEY in your environment')
def fetch_key():
 api_key=read_api_key()
 if api_key=='test':
  return 'test'
 data={'api_key':api_key,'version':VERSION}
 try:
  logging.getLogger('py.warnings').setLevel(logging.ERROR)
  result=requests.post('%s/activate'%config.API_URL,json.dumps(data),verify=ogjVA)
  key_base64=json.loads(result.content)['key']
  cache_key_locally(api_key,key_base64)
 except ogjVG:
  key_base64=load_cached_key(api_key)
 finally:
  logging.getLogger('py.warnings').setLevel(logging.WARNING)
 decoded_key=to_str(base64.b64decode(key_base64))
 return decoded_key
def cache_key_locally(api_key,key_b64):
 configs=localstack_config.load_config_file()
 timestamp=ogjVs(ogjVC(now_utc()))
 key_raw=to_str(base64.b64decode(key_b64))
 for i in ogjVn(ogjVM(timestamp)):
  key_raw=str_insert(key_raw,i*2,timestamp[i])
 key_b64=to_str(base64.b64encode(to_bytes(key_raw)))
 configs['cached_key']={'timestamp':ogjVC(timestamp),'key_hash':md5(api_key),'key':key_b64}
 save_file(localstack_config.CONFIG_FILE_PATH,json.dumps(configs))
 return configs
def load_cached_key(api_key):
 configs=localstack_config.load_config_file()
 cached_key=configs['cached_key']
 if cached_key.get('key_hash')!=md5(api_key):
  raise ogjVG('Cached key was created for a different API key')
 now=now_utc()
 if(now-cached_key['timestamp'])>MAX_KEY_CACHE_DURATION_SECS:
  raise ogjVG('Cached key expired')
 timestamp=ogjVs(cached_key['timestamp'])
 key_raw=to_str(base64.b64decode(cached_key['key']))
 for i in ogjVn(ogjVM(timestamp)):
  assert key_raw[i]==timestamp[i]
  key_raw=str_remove(key_raw,i)
 key_b64=to_str(base64.b64encode(to_bytes(key_raw)))
 return key_b64
def generate_aes_cipher(key):
 key=to_bytes(key)
 return pyaes.AESModeOfOperationCBC(key,iv='\0'*16)
def decrypt_file(source,target,key):
 cipher=generate_aes_cipher(key)
 raw=load_file(source,mode='rb')
 decrypter=pyaes.Decrypter(cipher)
 decrypted=decrypter.feed(raw)
 decrypted+=decrypter.feed()
 decrypted=decrypted.partition(b'\0')[0]
 decrypted=to_str(decrypted)
 save_file(target,content=decrypted)
def decrypt_files(key):
 files=[]
 for folder in PROTECTED_FOLDERS:
  for subpath in('*.py.enc','**/*.py.enc'):
   for f in glob.glob('%s/localstack_ext/%s/%s'%(ROOT_FOLDER,folder,subpath)):
    files.append(f)
 def _decrypt(f):
  target=f[:-4]
  if not os.path.exists(target):
   decrypt_file(f,target,key)
 parallelize(_decrypt,files)
def cleanup_environment():
 excepted_files=r'.*/services/((edge)|(dns_server)|(__init__))\.py'
 for folder in PROTECTED_FOLDERS:
  for subpath in('*.py.enc','**/*.py.enc'):
   for f in glob.glob('%s/localstack_ext/%s/%s'%(ROOT_FOLDER,folder,subpath)):
    target=f[:-4]
    if not re.match(excepted_files,target):
     for delete_file in(target,'%sc'%target):
      if os.path.exists(delete_file):
       os.remove(delete_file)
def prepare_environment():
 class OnClose(ogjVf):
  def __exit__(self,*args,**kwargs):
   if not ENV_PREPARED.get('finalized'):
    cleanup_environment()
   ENV_PREPARED['finalized']=ogjVQ
  def __enter__(self,*args,**kwargs):
   pass
 if not ENV_PREPARED.get('finalized'):
  try:
   key=fetch_key()
   if key!='test':
    decrypt_files(key)
    LOG.info('Successfully activated API key')
  except ogjVG:
   if config.REQUIRE_PRO:
    LOG.error('Unable to activate API key, but $REQUIRE_PRO is configured - quitting.')
    sys.exit(1)
   pass
 return OnClose()
def get_auth_headers():
 configs=localstack_config.load_config_file()
 login_info=configs.get('login')
 if not login_info:
  raise ogjVG('Please log in first')
 auth_token=login_info['token']
 if not auth_token.startswith('%s '%login_info['provider']):
  auth_token='%s %s'%(login_info['provider'],auth_token)
 headers={'authorization':auth_token}
 return headers
# Created by pyminifier (https://github.com/liftoff/pyminifier)
